/*
 * Copyright (c) 2003 Sun Microsystems, Inc.  All Rights Reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * Redistribution of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 
 * Redistribution in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * Neither the name of Sun Microsystems, Inc. or the names of
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * 
 * This software is provided "AS IS," without a warranty of any kind.
 * ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES,
 * INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED.
 * SUN MICROSYSTEMS, INC. ("SUN") AND ITS LICENSORS SHALL NOT BE LIABLE
 * FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.  IN NO EVENT WILL
 * SUN OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA,
 * OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR
 * PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE,
 * EVEN IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * 
 * You acknowledge that this software is not designed or intended for use
 * in the design, construction, operation or maintenance of any nuclear
 * facility.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <config.h>
#include <ipmitool/ipmi_intf.h>
#include <ipmitool/ipmi.h>

#ifdef IPMI_INTF_OPEN
extern struct ipmi_intf ipmi_open_intf;
#endif
#ifdef IPMI_INTF_IMB
extern struct ipmi_intf ipmi_imb_intf;
#endif
#ifdef IPMI_INTF_LIPMI
extern struct ipmi_intf ipmi_lipmi_intf;
#endif
#ifdef IPMI_INTF_LAN
extern struct ipmi_intf ipmi_lan_intf;
#endif
#ifdef IPMI_INTF_LANPLUS
extern struct ipmi_intf ipmi_lanplus_intf;
#endif

struct ipmi_intf * ipmi_intf_table[] = {
#ifdef IPMI_INTF_OPEN
	&ipmi_open_intf,
#endif
#ifdef IPMI_INTF_IMB
	&ipmi_imb_intf,
#endif
#ifdef IPMI_INTF_LIPMI
	&ipmi_lipmi_intf,
#endif
#ifdef IPMI_INTF_LAN
	&ipmi_lan_intf,
#endif
#ifdef IPMI_INTF_LANPLUS
	&ipmi_lanplus_intf,
#endif
	NULL
};

void ipmi_intf_print(void)
{
	struct ipmi_intf ** intf;
	int def = 1;

	printf("Interfaces:\n");

	for (intf = ipmi_intf_table; intf && *intf; intf++) {
		printf("\t%-12s %s", (*intf)->name, (*intf)->desc);
		if (def) {
			printf(" [default]");
			def = 0;
		}
		printf("\n");
	}
	printf("\n");
}

/* Load an interface from the interface table above
 * If no interface name is given return first entry
 */
struct ipmi_intf * ipmi_intf_load(char * name)
{
	struct ipmi_intf ** intf;
	struct ipmi_intf * i;

	if (!name) {
		i = ipmi_intf_table[0];
		if (i->setup && (i->setup(i) < 0)) {
			printf("ERROR: Unable to setup interface %s\n", name);
			return NULL;
		}
		return i;
	}

	for (intf = ipmi_intf_table; intf && *intf ; intf++) {
		i = *intf;
		if (!strncmp(name, i->name, strlen(name))) {
			if (i->setup && (i->setup(i) < 0)) {
				printf("ERROR: Unable to setup interface %s\n", name);
				return NULL;
			}
			return i;
		}
	}
	return NULL;
}

void ipmi_intf_session_set_hostname(struct ipmi_intf * intf, char * hostname)
{
	if (intf && intf->session) {
		memset(intf->session->hostname, 0, 16);
		if (hostname)
			memcpy(intf->session->hostname, hostname, min(strlen(hostname), 64));
	}
}

void ipmi_intf_session_set_username(struct ipmi_intf * intf, char * username)
{
	if (intf && intf->session) {
		memset(intf->session->username, 0, 16);
		if (username)
			memcpy(intf->session->username, username, min(strlen(username), 16));
	}
}

void ipmi_intf_session_set_password(struct ipmi_intf * intf, char * password)
{
	if (intf && intf->session) {
		memset(intf->session->authcode, 0, sizeof(intf->session->authcode));
		intf->session->password = 0;
		if (password) {
			intf->session->password = 1;
			memset(intf->session->authcode, 0, IPMI_AUTHCODE_BUFFER_SIZE);
			memcpy(intf->session->authcode,
			       password,
			       min(strlen(password), IPMI_AUTHCODE_BUFFER_SIZE));
		}
	}
}

void ipmi_intf_session_set_privlvl(struct ipmi_intf * intf, unsigned char level)
{
	if (intf && intf->session)
		intf->session->privlvl = level;
}

void ipmi_intf_session_set_port(struct ipmi_intf * intf, int port)
{
	if (intf && intf->session)
		intf->session->port = port;
}

void ipmi_intf_session_set_authtype(struct ipmi_intf * intf, unsigned char authtype)
{
	if (intf && intf->session)
		intf->session->authtype_set = authtype;
}

void ipmi_cleanup(struct ipmi_intf * intf)
{
	ipmi_sdr_list_empty(intf);
}
